import logging
from typing import cast

from fastapi import APIRouter, Depends, Path

from id_broker_common.kelvin import get_kelvin_obj, kelvin_session
from id_broker_common.utils import remove_prefix
from self_disclosure_plugin.dependencies import (
    connecting_user_pseudonym_from_token,
    extract_members,
    get_connected_user,
    get_service_provider_attribute,
    service_provider_from_token,
    token_dep,
)
from self_disclosure_plugin.models import CountedSchoolclass, NoStarStr, User, UsersClasses
from ucsschool.apis.opa import OPAClient, opa_instance
from ucsschool.apis.utils import get_logger
from ucsschool.kelvin.client import SchoolClass, SchoolClassResource, Session, User as KelvinUser

router = APIRouter(tags=["users"], dependencies=[Depends(token_dep)])


@router.get("/{id}/metadata", response_model=User)
async def user_metadata(
    pseudonym_of_requested_user: NoStarStr = Path(
        ...,
        alias="id",
        description="Pseudonym of user for which to retrieve data.",
        title="Pseudonym",
    ),
    pseudonym_of_connecting_user: str = Depends(connecting_user_pseudonym_from_token),
    connected_user: KelvinUser = Depends(get_connected_user),
    policy_instance: OPAClient = Depends(opa_instance),
    logger: logging.Logger = Depends(get_logger),
) -> User:
    logger.debug(
        "pseudonym_of_requested_user=%r pseudonym_of_connecting_user=%r connected_user=%r",
        pseudonym_of_requested_user,
        pseudonym_of_connecting_user,
        connected_user,
    )
    await policy_instance.check_policy_true_or_raise(
        "self_disclosure_plugin/users_self_info",
        {
            "sub": pseudonym_of_connecting_user,
            "user_id": pseudonym_of_requested_user,
            "user_roles": connected_user.roles,
        },
    )
    school_authority_prefix = f"{connected_user.source_uid}-"
    return User(
        user_id=pseudonym_of_requested_user,
        username=remove_prefix(connected_user.name, school_authority_prefix),
        firstname=connected_user.firstname,
        lastname=connected_user.lastname,
        type=", ".join(connected_user.roles),  # TODO what exactly to use here?
        school_id=remove_prefix(connected_user.school, school_authority_prefix),
    )


@router.get("/{id}/groups", response_model=UsersClasses)
async def user_groups(
    pseudonym_of_requested_user: NoStarStr = Path(
        ...,
        alias="id",
        description="Pseudonym of user for which to retrieve data.",
        title="Pseudonym",
    ),
    pseudonym_of_connecting_user: str = Depends(connecting_user_pseudonym_from_token),
    connected_user: KelvinUser = Depends(get_connected_user),
    service_provider: str = Depends(service_provider_from_token),
    policy_instance: OPAClient = Depends(opa_instance),
    session: Session = Depends(kelvin_session),
    logger: logging.Logger = Depends(get_logger),
) -> UsersClasses:
    logger.debug(
        "pseudonym_of_requested_user=%r pseudonym_of_connecting_user=%r connected_user=%r "
        "connected_user.school_classes=%r, service_provider=%r",
        pseudonym_of_requested_user,
        pseudonym_of_connecting_user,
        connected_user,
        connected_user.school_classes,
        service_provider,
    )
    await policy_instance.check_policy_true_or_raise(
        "self_disclosure_plugin/users_self_info",
        {"sub": pseudonym_of_connecting_user, "user_id": pseudonym_of_requested_user},
    )

    sp_attribute = await get_service_provider_attribute(service_provider)
    groups = []
    for school, classes in connected_user.school_classes.items():
        for classname in classes:
            school_class = cast(
                SchoolClass,
                await get_kelvin_obj(
                    SchoolClassResource(session=session),
                    name=classname,
                    school=school,
                ),
            )
            class_members = await extract_members(
                school_class, service_provider, session, list_teachers=False
            )
            groups.append(
                CountedSchoolclass(
                    group_id=school_class.udm_properties[sp_attribute],
                    name=classname,
                    student_count=len(class_members.students),
                )
            )

    return UsersClasses(groups=groups)
